/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2025 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#define __PYNIC_LOG_IMPLIMENT__
#include <logger.h>
#include <ge_interface/ge_interface.h>

#ifdef __USE_UCOS2__
#include <ucos_ii.h>
#endif

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
#define TASK_SIZE 128
#define STACK_NUM 6
static OS_STK stacks[STACK_NUM][TASK_SIZE];
#define GET_STACK_TOP(number) ((OS_STK*)&stacks[number][TASK_SIZE - 1])
#define GET_STACK_BOTTOM(number) ((OS_STK*)&stacks[number][0])
#define _STR(str) # str
#define STR(str) _STR(str)
#define CREATE_TASK(func, args, stack_num, prio) \
  OSTaskCreateExt(func, args, GET_STACK_TOP(stack_num), prio, stack_num, \
  GET_STACK_BOTTOM(stack_num), TASK_SIZE, STR(func), OS_TASK_OPT_STK_CHK);

static void T_ledR(void* args) {
  (void) args;
  const int delay_s = 1;
  while (1) {
    GE_LED_OFF(R);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
    GE_LED_ON(R);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
  }
}

static void T_ledG(void* args) {
  (void) args;
  const int delay_s = 2;
  while (1) {
    GE_LED_OFF(G);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
    GE_LED_ON(G);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
  }
}

static void T_ledB(void* args) {
  (void) args;
  const int delay_s = 3;
  while (1) {
    GE_LED_OFF(B);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
    GE_LED_ON(B);
    OSTimeDlyHMSM(0, 0, delay_s, 0);
  }
}

static void game_main(void* args) {
  (void) args;
#define RED         0xf800
#define GREEN       0x07e0
#define BLUE        0x001f
#define WHITE       0xffff
#define BLACK       0x0000
#define YELLOW      0xFFE0
#define GRAY0       0xEF7D
#define GRAY1       0x8410
#define GRAY2       0x4208
  ge_input_event_t ievent;
  ge_render_rect_t player = {
    .pos.x = 60,
    .pos.y = 60,
    .size.x = 8,
    .size.y = 8,
  };
  int speed = 5;

  ge_render.init_func(&ge_render, NULL);
  while (1) {
    input_btn_func(&ge_input);
    while (ge_input.func_recv(&ge_input, &ievent) == 0) {
      switch (ievent.num) {
        case GE_ITYPE_KEY_1:
          player.pos.x += speed;
          break;
        case GE_ITYPE_KEY_2:
          player.pos.y -= speed;
          break;
        case GE_ITYPE_KEY_3:
          player.pos.y += speed;
          break;
        case GE_ITYPE_KEY_4:
          player.pos.x -= speed;
          break;
        default:
          LOG_INFO("%d pressd", ievent.num);
          break;
      }
    }
    ge_render.func_draw_rect(&ge_render, &(ge_render_rect_t) {
      .pos = {0, 0},
      .size = ge_render.screen_size
    }, WHITE);
    ge_render.func_draw_rect(&ge_render, &player, RED);
    ge_render.func_flush(&ge_render);
    OSTimeDlyHMSM(0, 0, 0, 33);
  }
}

void start_task(void *args) {
  (void)args;
  OS_CPU_SR cpu_sr=0;
  OS_ENTER_CRITICAL();
  CREATE_TASK(game_main, NULL, 4, 9);
  CREATE_TASK(T_ledR, NULL, 1, 5);
  CREATE_TASK(T_ledG, NULL, 2, 6);
  CREATE_TASK(T_ledB, NULL, 3, 7);
	OSTaskSuspend(10);
	OS_EXIT_CRITICAL();
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{

  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  /* USER CODE BEGIN 2 */
  INIT_LOGGER();
  OSInit();
  OSTaskCreate(start_task,(void *)0, GET_STACK_TOP(0), 10);//创建起始任务
  OSStart();
  // Never come here
  Error_Handler();
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  HAL_GPIO_TogglePin(SOND_GPIO_Port, SOND_Pin);
  HAL_Delay(100);
  HAL_GPIO_TogglePin(SOND_GPIO_Port, SOND_Pin);
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
